# Supports Organizer.rb
=begin rdoc
= Three Line Tools
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

== License
Portions of the code Copyright 2005-2007, Google, Inc. and can be found in utilities.rb that comes with Sketchup.
The rest is Copyright 2009, Chris Fullmer
You may not alter, publish, market, distribute, give, transfer, sell or sublicense the Scripts or any part of this Script.

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.sketchucation.com
Name:: Three Line Tools
Version:: 2.1
SU Version:: I developed this on SketchUp version 7, but older versions should work.
Date:: March 09, 2009
Description:: Three tools to help make vertical or normal lines by clicking on a point, then enter the desired line length or elevation.
Usage:: Activate it at "Plugins>Chris Fullmer Tools>3 Line Tools".  There are 3 tools available.  Lines by height creates vertical lines with the user input length. Vertical lines by elevation creates vertical lines by the desired elevation. Normal Lines creates lines that follow the normals of the face, edge, or vertex that is selected.
Requires:: No additional scripts
History::
1.0:: 2009-02-22
   * Original release
1.5:: 2009-02-26
   * Added 2 more tools (3 total) and a submenu to organize them all
   * Vertical lines by height lets you add a vertical by specifying its total height (can be negative).
   * Vertical lines by elevation lets you add a vertical by specifying its ending elevation (can be negative).
   * Lines on Normals lets you add a line that will follow the normal of the face, edge, or vertex clicked on.
   * Added "Undo" support. 
1.6:: 2009-02-28
   * Added VCB support so now you can enter a vlue and click repeatedly and not have to hit enter constantly.
2.0:: 2009-02-28
   * Made some icons and added a toolbar.
   * Now this plugin registers itself with the SketchUp Extension manager so you can disable it if you don't want it in your menu.
2.1:: 2009-03-09
   * Organizer.rb support. If you place this script into an organizer directory, it will appear in the correct submenu.  If its just in the plugins folder, it will appear in "Chris Fullmer Tools" submenu of the Plugins Menu."
=end

require 'sketchup.rb'

# This is the tool that makes vertical lines of an input height.
class Clf_Vertical_lines_height
@@last_height = ""

def activate
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
	@start_pos = []
	@end_pos = []
	if @@last_height.to_s == ""
		@@last_height = 100
	end
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Length"
end

def resume view
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Length"
end

 def enableVCB?
   return true
 end

  def onUserText(text, view)
	@@last_height = text.to_l
	@end_pos = [@start_pos.x, @start_pos.y, @@last_height.to_l]
	if (@last_line[0])
		(@last_line[0]).erase!
	end
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	@@total_height = @@last_height + @start_pos.z.to_l
	@end_pos = [@start_pos.x, @start_pos.y, @@total_height.to_l]
	make_line
 end
 
def onMouseMove(flags, x, y, view)
    pos = @ip.position
	Sketchup::set_status_text("#{pos.x}, #{pos.y}, #{pos.z}")    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;           
        if( changed and (@ip.display? or @displayed) )
            view.invalidate
        end
    end
end

# onLButtonUp is called when the user releases the left mouse button
def onLButtonUp(flags, x, y, view)
	@start_pos = @ip.position
    Sketchup::set_status_text $uStrings.GetString("Left button up at") + " (#{@start_pos.x}, #{@start_pos.y}, #{@start_pos.z})"
	@@total_height = @@last_height + @start_pos.z.to_l
	@end_pos = [@start_pos.x, @start_pos.y, @@total_height.to_l]
	Sketchup.active_model.start_operation "Line by Height"
	make_line
	Sketchup.active_model.commit_operation 
end

def make_line
	model = Sketchup.active_model
	entities = model.active_entities
	@last_line = entities.add_edges @start_pos, @end_pos
end #make_line

def draw(view)
    if( @ip.valid? && @ip.display? )
        @ip.draw view
        @displayed = true
    else
        @displayed = false
    end
end
end


# This is the tool that makes vertical lines to an input elevation.
class Clf_Vertical_lines_elev
@@last_height = ""

def activate
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
	@start_pos = []
	@end_pos = []
	if @@last_height.to_s == ""
		@@last_height = 100
	end
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Elevation"
end

 def enableVCB?
   return true
 end
 
 def resume view
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Elevation"
end
 
 def onUserText(text, view)
	@@last_height = text.to_l
	@end_pos = [@start_pos.x, @start_pos.y, @@last_height.to_l]
	if (@last_line[0])
		(@last_line[0]).erase!
	end
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	make_line
 end

 
def onMouseMove(flags, x, y, view)
    pos = @ip.position
   Sketchup::set_status_text("#{pos.x}, #{pos.y}, #{pos.z}")    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;           
        if( changed and (@ip.display? or @displayed) )
            view.invalidate
        end
    end
end


# onLButtonUp is called when the user releases the left mouse button
def onLButtonUp(flags, x, y, view)
	@start_pos = @ip.position
    Sketchup::set_status_text $uStrings.GetString("Left button up at") + " (#{@start_pos.x}, #{@start_pos.y}, #{@start_pos.z})"
	@end_pos = [@start_pos.x, @start_pos.y, @@last_height.to_l]
	Sketchup.active_model.start_operation "Line by Elevation"
	make_line
	Sketchup.active_model.commit_operation 
end

def make_line
	model = Sketchup.active_model
	entities = model.active_entities
	@last_line = entities.add_edges @start_pos, @end_pos
end #make_line


def draw(view)
    if( @ip.valid? && @ip.display? )
        @ip.draw view
        @displayed = true
    else
        @displayed = false
    end
end
end


# This is the tool that makes lines following the normals of the point where its made.
class Clf_Normal_lines_Height
@@last_height = ""

def activate
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
	@start_pos = []
	@end_pos = []
	if @@last_height.to_s == ""
		@@last_height = 100
	end
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Length"
end

def resume view
	Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
	Sketchup.vcb_label = "Line Length"
end

 def enableVCB?
   return true
 end

def onUserText(text, view)
		@@last_height = text.to_l
		endpoint_maker @faces
		if (@last_line[0])
			(@last_line[0]).erase!
		end
		Sketchup.set_status_text @@last_height.to_l, SB_VCB_VALUE
		make_line
end # onUserText

def onMouseMove(flags, x, y, view)
    pos = @ip.position
   Sketchup::set_status_text("#{pos.x}, #{pos.y}, #{pos.z}")    
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;           
        if( changed and (@ip.display? or @displayed) )
            view.invalidate
        end
    end
end

# This method makes a line between @start_pos and @end_pos.  
# @start_pos is set when the user clicks. @end_pos is determined in the endpoint_maker method
def make_line
model = Sketchup.active_model
entities = model.active_entities
@last_line = entities.add_edges @start_pos, @end_pos
end #make_line

# This method takes an array of faces as its input.  It then finds the normals for each face, and finds an averge
# normal for all faces (or a single face if the user clicks on a single face).
def endpoint_maker faces
	a = 0
	b = 0
	c = 0
	faces.each do |e| # Gets the xyz values for the normals and adds them up.
		a = e.normal.x + a
		b = e.normal.y + b
		c = e.normal.z + c
	end		
	x = a / faces.length # Takes the sum and divivdes by the number of faces, providing the average values
	y = b / faces.length
	z = c / faces.length
	vector = [x,y,z] # This is the new normal vector that will be used to make the end point for our line.
	@end_pos = @start_pos.offset(vector, @@last_height) # set the value for @end_pos using the @start_pos, the normal vector, and user input length
end # endpoint_maker

# onLButtonUp is called when the user releases the left mouse button
def onLButtonUp(flags, x, y, view)
	@start_pos = @ip.position
    Sketchup::set_status_text $uStrings.GetString("Left button up at") + " (#{@start_pos.x}, #{@start_pos.y}, #{@start_pos.z})"
	vertex = @ip.vertex
	face = @ip.face
	edge = @ip.edge
	
	if(vertex)
		face = nil
		@faces = vertex.faces
		endpoint_maker @faces
	end
	
	if(edge)
		face = nil		
		@faces = edge.faces
		endpoint_maker @faces
	end

	if(face)
		@faces = [face]
		endpoint_maker @faces
	end	
	
	# This code controls the "undo" for the script.  
	if @undo == 1
		Sketchup.active_model.commit_operation
		@undo == 0
	end
	if @undo == 0
		Sketchup.active_model.start_operation "Normal Line"
		@undo = 1
	end
	make_line
end

def draw(view)
    if( @ip.valid? && @ip.display? )
        @ip.draw view
        @displayed = true
    else
        @displayed = false
    end
end
end

if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		#organizer.add_item("OnScreen Display") { Sketchup.active_model.select_tool Clf_OnScreen_Display.new }
		clf_line_tools_menu = organizer.add_submenu("3 Line Tools")
		
		toolbar = UI::Toolbar.new "3 Line Tools"
		cmd1 = UI::Command.new("Vertical Lines by Length") { Sketchup.active_model.select_tool Clf_Vertical_lines_height.new }
		cmd2 = UI::Command.new("Vertical Lines by Elevation") { Sketchup.active_model.select_tool Clf_Vertical_lines_elev.new }
		cmd3 = UI::Command.new("Line on Normals") { Sketchup.active_model.select_tool Clf_Normal_lines_Height.new }
		
		clf_line_tools_menu.add_item cmd1
		clf_line_tools_menu.add_item cmd2
		clf_line_tools_menu.add_item cmd3
		
		cmd1.small_icon = "line_length_16.png"
		cmd1.large_icon = "line_length.png"
		cmd2.small_icon = "line_elev_16.png"
		cmd2.large_icon = "line_elev.png"
		cmd3.small_icon = "line_normals_16.png"
		cmd3.large_icon = "line_normals.png"
		
		cmd1.tooltip = "Vertical Lines by Length"
		cmd2.tooltip = "Vertical Lines by Elevation"
		cmd3.tooltip = "Line on Normals"
		toolbar = toolbar.add_item cmd1
		toolbar = toolbar.add_item cmd2
		toolbar = toolbar.add_item cmd3
		toolbar.show
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
	    clf_line_tools_menu = $clf_tools_menu.add_submenu("3 Line Tools")
		
		toolbar = UI::Toolbar.new "3 Line Tools"
		cmd1 = UI::Command.new("Vertical Lines by Length") { Sketchup.active_model.select_tool Clf_Vertical_lines_height.new }
		cmd2 = UI::Command.new("Vertical Lines by Elevation") { Sketchup.active_model.select_tool Clf_Vertical_lines_elev.new }
		cmd3 = UI::Command.new("Line on Normals") { Sketchup.active_model.select_tool Clf_Normal_lines_Height.new }
		
		clf_line_tools_menu.add_item cmd1
		clf_line_tools_menu.add_item cmd2
		clf_line_tools_menu.add_item cmd3
		
		cmd1.small_icon = "line_length_16.png"
		cmd1.large_icon = "line_length.png"
		cmd2.small_icon = "line_elev_16.png"
		cmd2.large_icon = "line_elev.png"
		cmd3.small_icon = "line_normals_16.png"
		cmd3.large_icon = "line_normals.png"
		
		cmd1.tooltip = "Vertical Lines by Length"
		cmd2.tooltip = "Vertical Lines by Elevation"
		cmd3.tooltip = "Line on Normals"
		toolbar = toolbar.add_item cmd1
		toolbar = toolbar.add_item cmd2
		toolbar = toolbar.add_item cmd3
		toolbar.show
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)